from org.telegram.ui.ActionBar import BaseFragment as BaseFragment, Theme as Theme
from typing import Callable

class BulletinHelper:
    DURATION_SHORT: int
    DURATION_LONG: int
    DURATION_PROLONG: int
    @classmethod
    def show_info(cls, message: str, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_error(cls, message: str, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_success(cls, message: str, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_simple(cls, text: str, icon_res_id: int, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_two_line(cls, title: str, subtitle: str, icon_res_id: int, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_with_button(cls, text: str, icon_res_id: int, button_text: str, on_click: Callable[[], None] | None, fragment: BaseFragment | None = None, duration: int = ...): ...
    @classmethod
    def show_undo(cls, text: str, on_undo: Callable[[], None], on_action: Callable[[], None] | None = None, subtitle: str | None = None, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_copied_to_clipboard(cls, message: str | None = None, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_link_copied(cls, is_private_link_info: bool = False, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_file_saved_to_gallery(cls, is_video: bool = False, amount: int = 1, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_file_saved_to_downloads(cls, file_type_enum_name: str = 'UNKNOWN', amount: int = 1, fragment: BaseFragment | None = None): ...



from _typeshed import Incomplete
from base_plugin import HookResult
from typing import Any

class PluginsManager:
    INJECTED_MODULE_DICT: Incomplete
    @classmethod
    def validate_plugin_from_file(cls, file_path: str) -> dict[str, str]: ...
    @classmethod
    def load_plugin(cls, file_name: str, file_path: str | None = None) -> str | None: ...
    @classmethod
    def load_plugin_from_file(cls, file_path: str) -> str | None: ...
    @classmethod
    def unload_plugin(cls, plugin_id: str, keep_entry: bool = False) -> bool: ...
    @classmethod
    def delete_plugin(cls, plugin_id: str) -> bool: ...
    @classmethod
    def set_plugin_enabled(cls, plugin_id: str, enabled: bool) -> bool: ...
    @classmethod
    def init(cls, plugins_directory: str): ...
    @classmethod
    def shutdown(cls) -> None: ...
    @classmethod
    def cleanup_plugin(cls, plugin_id: str): ...
    @classmethod
    def load_settings(cls, plugin_id: str) -> list[Any] | None: ...
    @classmethod
    def get_plugin_setting(cls, plugin_id: str, key: str, default: Any = None) -> Any: ...
    @classmethod
    def set_plugin_setting(cls, plugin_id: str, key: str, value: Any): ...
    @classmethod
    def execute_event(cls, event_type: str): ...
    @classmethod
    def execute_pre_request_hook(cls, interested_plugin_ids: list[str], request_name: str, account: int, request: Any) -> HookResult: ...
    @classmethod
    def execute_post_request_hook(cls, interested_plugin_ids: list[str], request_name: str, account: int, response: Any, error: Any) -> HookResult: ...
    @classmethod
    def execute_on_update_hook(cls, interested_plugin_ids: list[str], update_name: str, account: int, update: Any) -> HookResult: ...
    @classmethod
    def execute_on_updates_hook(cls, interested_plugin_ids: list[str], container_name: str, account: int, updates: Any) -> HookResult: ...
    @classmethod
    def execute_on_send_message_hook(cls, interested_plugin_ids: list[str], account: int, params: Any) -> HookResult: ...



import argparse
import ast
import json
import logging
import os
import socket
import subprocess
import sys
import threading
import time
from dataclasses import dataclass
from threading import Lock, Event
from typing import Dict, Any, Optional, List, Tuple


@dataclass
class PluginMetadata:
    id: str
    name: str
    description: str
    author: str
    version: str
    icon: str
    min_version: str


@dataclass
class PendingRequest:
    request_id: int
    action: str
    event: Event
    response: Optional[Dict[str, Any]] = None
    error: Optional[str] = None


class AdbManager:
    def __init__(self):
        self.logger: logging.Logger = logging.getLogger("AdbManager")

    def run_command(self, command: List[str]) -> Optional[str]:
        try:
            result = subprocess.run(
                ["adb"] + command, check=True, capture_output=True, text=True
            )
            self.logger.debug(f"ADB command successful: {' '.join(command)}")
            return result.stdout.strip()
        except subprocess.CalledProcessError as e:
            self.logger.error(f"ADB command failed: {e}")
            self.logger.error(f"Error output: {e.stderr}")
            return None

    def start_server(self):
        self.logger.info("Starting ADB server...")
        self.run_command(["start-server"])

    def wait_for_device(self):
        self.logger.info("Waiting for device to connect...")
        self.run_command(["wait-for-device"])
        self.logger.info("Device connected")

    def forward_port(self, local_port: int, remote_port: int) -> bool:
        self.logger.info(f"Forwarding port {local_port} to {remote_port}...")
        result = self.run_command(
            ["forward", f"tcp:{local_port}", f"tcp:{remote_port}"]
        )
        return result is not None

    def reverse_port(self, local_port: int, remote_port: int) -> bool:
        self.logger.info(f"Reverse forwarding port {remote_port}...")
        result = self.run_command(
            ["reverse", f"tcp:{local_port}", f"tcp:{remote_port}"]
        )
        return result is not None

    def setup_device(self, debug_mode: bool = False) -> bool:
        self.start_server()
        self.wait_for_device()

        main_success = self.forward_port(42690, 42690)

        if debug_mode:
            debug_success = self.forward_port(5678, 5678)
            return main_success and debug_success

        return main_success


class DeviceConnection:
    def __init__(
        self,
        host: str = "127.0.0.1",
        port: int = 42690,
        debug_enabled: bool = False,
        debug_host: str = "127.0.0.1",
        debug_port: int = 5678,
        retry_delay: int = 2,
        response_timeout: int = 30,
    ):
        self.host: str = host
        self.port: int = port
        self.debug_enabled: bool = debug_enabled
        self.debug_host: str = debug_host
        self.debug_port: int = debug_port
        self.retry_delay: int = retry_delay
        self.response_timeout: int = response_timeout
        self.socket: Optional[socket.socket] = None
        self.connected: bool = False
        self.logger: logging.Logger = logging.getLogger("DeviceConnection")
        self.ping_thread: Optional[threading.Thread] = None
        self.response_thread: Optional[threading.Thread] = None
        self.running: bool = True
        self.request_id: int = 1

        self.pending_requests: Dict[int, PendingRequest] = {}
        self.requests_lock: Lock = Lock()

        self.response_buffer: bytes = b""
        self.buffer_lock: Lock = Lock()

    def connect(self) -> bool:
        if self.connected and self.socket:
            return True

        attempt: int = 1
        while self.running:
            try:
                self.logger.info(f"Connecting to device (attempt {attempt})...")
                self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                self.socket.settimeout(1.0)  # Set timeout for non-blocking reads
                self.socket.connect((self.host, self.port))
                self.connected = True
                self.logger.info("Connected to device")

                if not self.response_thread or not self.response_thread.is_alive():
                    self.start_response_thread()

                if self.debug_enabled:
                    self.stop_debugger()
                    self.logger.info("Sent stop_debugger message")
                    time.sleep(0.5)
                    self.setup_debugger()

                if not self.ping_thread or not self.ping_thread.is_alive():
                    self.start_ping_thread()

                return True
            except Exception as e:
                self.logger.error(f"Connection attempt {attempt} failed: {e}")
                if self.socket:
                    self.socket.close()
                    self.socket = None
                self.connected = False

                self.logger.info(f"Retrying in {self.retry_delay} seconds...")
                time.sleep(self.retry_delay)
                attempt += 1

        return False

    def disconnect(self):
        self.running = False

        with self.requests_lock:
            for request in self.pending_requests.values():
                request.error = "Connection closed"
                request.event.set()
            self.pending_requests.clear()

        if self.socket:
            try:
                self.socket.close()
            except Exception as e:
                self.logger.error(f"Error closing connection: {e}")
            finally:
                self.socket = None
                self.connected = False
                self.logger.info("Disconnected from device")

    def start_response_thread(self):
        def response_reader():
            while self.running and self.connected:
                try:
                    data = self.socket.recv(4096)
                    if not data:
                        self.logger.error("Connection closed by server")
                        self.connected = False
                        break

                    with self.buffer_lock:
                        self.response_buffer += data
                        self._process_response_buffer()

                except socket.timeout:
                    continue
                except Exception as e:
                    if self.running:
                        self.logger.error(f"Error reading response: {e}")
                        self.connected = False
                    break

        self.response_thread = threading.Thread(target=response_reader, daemon=True)
        self.response_thread.start()
        self.logger.debug("Response reader thread started")

    def _process_response_buffer(self):
        buffer_str = self.response_buffer.decode("utf-8", errors="replace")
        pos = 0

        while pos < len(buffer_str):
            while pos < len(buffer_str) and buffer_str[pos].isspace():
                pos += 1

            if pos >= len(buffer_str):
                break

            try:
                json_obj, end_pos = json.JSONDecoder().raw_decode(buffer_str[pos:])
                self._handle_response(json_obj)
                pos += end_pos
            except json.JSONDecodeError:
                break

        self.response_buffer = buffer_str[pos:].encode("utf-8")

    def _handle_response(self, response: Dict[str, Any]):
        request_id = response.get("#")
        if request_id is None:
            self.logger.warning(f"Received response without request ID: {response}")
            return

        with self.requests_lock:
            pending_request = self.pending_requests.get(request_id)
            if pending_request:
                pending_request.response = response
                pending_request.event.set()
                del self.pending_requests[request_id]
                self.logger.debug(f"Matched response for request {request_id}")
            else:
                self.logger.warning(
                    f"Received response for unknown request ID: {request_id}"
                )

    def ping(self) -> bool:
        response = self.send_message("ping")
        return response is not None

    def start_ping_thread(self):
        def ping_worker():
            while self.running and self.connected:
                try:
                    if not self.ping():
                        self.logger.error("Ping failed, connection may be lost")
                        break
                    time.sleep(10)
                except Exception as e:
                    self.logger.error(f"Error in ping thread: {e}")
                    break

        self.ping_thread = threading.Thread(target=ping_worker, daemon=True)
        self.ping_thread.start()
        self.logger.debug("Ping thread started")

    def send_message(
        self, action: str, arguments: Optional[Dict[str, Any]] = None
    ) -> Optional[Dict[str, Any]]:
        if not self.connected and not self.connect():
            return None

        with self.requests_lock:
            request_id = self.request_id
            self.request_id += 1

            pending_request = PendingRequest(
                request_id=request_id, action=action, event=Event()
            )
            self.pending_requests[request_id] = pending_request

        message = {
            "@": action,
            "#": request_id,
        }

        if arguments:
            message.update(arguments)

        try:
            data = json.dumps(message).encode("utf-8")
            self.socket.sendall(data)
            self.logger.debug(f"Sent message: {action} (ID: {request_id})")

            if pending_request.event.wait(timeout=self.response_timeout):
                if pending_request.error:
                    self.logger.error(
                        f"Request {request_id} failed: {pending_request.error}"
                    )
                    return None
                return pending_request.response
            else:
                with self.requests_lock:
                    if request_id in self.pending_requests:
                        del self.pending_requests[request_id]
                self.logger.error(
                    f"Request {request_id} timed out after {self.response_timeout}s"
                )
                return None

        except Exception as e:
            self.logger.error(f"Failed to send message: {e}")
            with self.requests_lock:
                if request_id in self.pending_requests:
                    del self.pending_requests[request_id]
            self.disconnect()
            if self.connect():
                return self.send_message(action, arguments)
            return None

    def get_plugins(self) -> Optional[Dict[str, Any]]:
        response = self.send_message("get_plugins")
        if response:
            response.pop("#", None)
            return response
        return None

    def write_plugin(self, plugin_id: str, content: str) -> bool:
        response = self.send_message(
            "write_plugin", {"plugin_id": plugin_id, "content": content}
        )
        return response is not None

    def reload_plugin(self, plugin_id: str) -> bool:
        response = self.send_message("reload_plugin", {"plugin_id": plugin_id})
        return response is not None

    def stop_debugger(self) -> bool:
        response = self.send_message("stop_debugger", {"platform": "vscode"})
        return response is not None

    def setup_debugger(self) -> bool:
        response = self.send_message(
            "start_debugger",
            {
                "host": self.debug_host,
                "port": self.debug_port,
                "platform": "vscode",
            },
        )
        return response is not None


# copied from plugins manager
def parse_metadata(content: str) -> Optional[PluginMetadata]:
    logger = logging.getLogger("metadata")
    metadata_dict = {}
    try:
        tree = ast.parse(content)
        for node in ast.iter_child_nodes(tree):
            if isinstance(node, ast.Assign):
                for target in node.targets:
                    if (
                        isinstance(target, ast.Name)
                        and target.id.startswith("__")
                        and target.id.endswith("__")
                    ):
                        if isinstance(node.value, ast.Constant):
                            metadata_dict[target.id[2:-2]] = node.value.value

        plugin_id = metadata_dict.get("id")
        plugin_name = metadata_dict.get("name")

        if not isinstance(plugin_id, str) or not plugin_id:
            raise ValueError(
                "Plugin metadata missing or invalid '__id__'. Must be a non-empty string."
            )
        if not isinstance(plugin_name, str) or not plugin_name:
            raise ValueError(
                "Plugin metadata missing or invalid '__name__'. Must be a non-empty string."
            )

        return PluginMetadata(
            id=plugin_id,
            name=plugin_name,
            description=metadata_dict.get("description"),
            author=metadata_dict.get("author"),
            version=metadata_dict.get("version"),
            icon=metadata_dict.get("icon"),
            min_version=metadata_dict.get("min_version"),
        )
    except Exception as e:
        logger.error(f"Error parsing plugin metadata from file: {e}")
        return None


class FileMonitor:
    def __init__(self, connection: DeviceConnection):
        self.connection: DeviceConnection = connection
        self.logger: logging.Logger = logging.getLogger("FileMonitor")
        self.file_metadata: Dict[str, Tuple[float, Optional[PluginMetadata]]] = {}
        self.running: bool = True

    def add_files(self, filenames: List[str]):
        for filename in filenames:
            if not os.path.isfile(filename):
                self.logger.error(f"File '{filename}' not found - skipping")
                continue

            content, metadata = self._read_file_and_metadata(filename)
            if metadata is None or not metadata.id:
                self.logger.error(
                    f"File '{filename}' has no valid plugin ID - skipping"
                )
                continue

            self.file_metadata[filename] = (os.path.getmtime(filename), metadata)

            self._upload_file(filename, content, metadata.id)
            self.logger.info(
                f"Added file '{filename}' with plugin ID '{metadata.id}' for monitoring"
            )

    def _read_file_and_metadata(
        self, filename: str
    ) -> Tuple[str, Optional[PluginMetadata]]:
        try:
            with open(filename, "r", encoding="utf-8") as f:
                content = f.read()

            metadata = parse_metadata(content)
            return content, metadata
        except Exception as e:
            self.logger.error(f"Error reading file '{filename}': {e}")
            return "", None

    def _upload_file(self, filename: str, content: str, plugin_id: str) -> bool:
        try:
            if self.connection.write_plugin(plugin_id, content):
                self.logger.info(f"Uploaded '{filename}' with plugin ID '{plugin_id}'")
                time.sleep(0.3)
                success = self.connection.reload_plugin(plugin_id)
                if success:
                    self.logger.info(f"Reloaded plugin '{plugin_id}'")
                else:
                    self.logger.warning(f"Failed to reload plugin '{plugin_id}'")
                return success
            self.logger.warning(f"Failed to upload '{filename}'")
            return False
        except Exception as e:
            self.logger.error(f"Error uploading file '{filename}': {e}")
            return False

    def start_monitoring(self):
        if not self.file_metadata:
            self.logger.warning("No valid files to monitor")
            return

        self.logger.info(f"Starting to monitor {len(self.file_metadata)} files")

        plugins = self.connection.get_plugins()
        if plugins:
            self.logger.info(f"Connected device has {len(plugins)} plugins loaded")

        try:
            while self.running:
                time.sleep(1)
                self._check_for_changes()
        except KeyboardInterrupt:
            self.logger.info("Monitoring stopped by user")
        finally:
            self.connection.stop_debugger()
            self.connection.disconnect()

    def _check_for_changes(self):
        for filename, (last_modified, metadata) in list(self.file_metadata.items()):
            try:
                if not os.path.isfile(filename):
                    self.logger.warning(
                        f"File '{filename}' no longer exists - skipping check"
                    )
                    continue

                current_modified = os.path.getmtime(filename)
                if current_modified != last_modified:
                    self.logger.info(f"File '{filename}' changed")

                    content, new_metadata = self._read_file_and_metadata(filename)

                    if new_metadata is None or not new_metadata.id:
                        self.logger.warning(
                            f"Modified file '{filename}' has no valid plugin ID - skipping update"
                        )
                        self.file_metadata[filename] = (current_modified, metadata)
                        continue

                    self._upload_file(filename, content, new_metadata.id)

                    self.file_metadata[filename] = (current_modified, new_metadata)

            except FileNotFoundError:
                self.logger.error(f"File '{filename}' not found during check")
            except PermissionError:
                self.logger.error(f"Permission denied when accessing '{filename}'")
            except Exception as e:
                self.logger.error(f"Error checking file '{filename}': {e}")


def parse_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        description="Monitor plugins' changes and sync them to Android device."
    )
    parser.add_argument(
        "files", nargs="+", help="One or more files to monitor for changes"
    )
    parser.add_argument("--debug", action="store_true", help="Enable PyCharm debugger")
    parser.add_argument(
        "--log-level",
        choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
        default="INFO",
        help="Set the logging level",
    )
    return parser.parse_args()


def setup_logging(level: str):
    numeric_level: int = getattr(logging, level.upper(), None)
    log_format: str = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logging.basicConfig(level=numeric_level, format=log_format)


def main():
    args = parse_arguments()
    setup_logging(args.log_level)
    logger = logging.getLogger("main")

    valid_files = [f for f in args.files if os.path.isfile(f)]
    if not valid_files:
        logger.error("None of the specified files exist. Exiting.")
        sys.exit(1)

    adb_manager = AdbManager()
    if not adb_manager.setup_device(args.debug):
        logger.error("Failed to set up ADB connection. Exiting.")
        sys.exit(1)

    connection = DeviceConnection(debug_enabled=args.debug)
    if not connection.connect():
        logger.error("Failed to establish connection to device. Exiting.")
        sys.exit(1)

    monitor = FileMonitor(connection)
    monitor.add_files(args.files)

    logger.info(
        f"Monitoring {len(args.files)} files for changes. Press Ctrl+C to stop."
    )
    monitor.start_monitoring()


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass



from _typeshed import Incomplete
from java import cast as cast
from org.telegram.messenger import DispatchQueue, R as R
from org.telegram.tgnet import TLObject as TLObject, TLRPC as TLRPC
from typing import Any

STAGE_QUEUE: str
GLOBAL_QUEUE: str
CACHE_CLEAR_QUEUE: str
SEARCH_QUEUE: str
PHONE_BOOK_QUEUE: str
THEME_QUEUE: str
EXTERNAL_NETWORK_QUEUE: str
PLUGINS_QUEUE: str

def get_queue_by_name(queue_name: str) -> DispatchQueue | None: ...
def run_on_queue(fn: callable, queue_name: str = ..., delay: int = 0): ...

class RequestCallback(Incomplete):
    def __init__(self, fn: callable) -> None: ...
    def run(self, response: TLObject, error: TLRPC.TL_error): ...

def send_request(request: Any, fn: callable) -> int: ...
def get_last_fragment(): ...
def get_account_instance(): ...
def get_messages_controller(): ...
def get_contacts_controller(): ...
def get_media_data_controller(): ...
def get_connections_manager(): ...
def get_location_controller(): ...
def get_notifications_controller(): ...
def get_messages_storage(): ...
def get_send_messages_helper(): ...
def get_file_loader(): ...
def get_secret_chat_helper(): ...
def get_download_controller(): ...
def get_notifications_settings(): ...
def get_notification_center(): ...
def get_media_controller(): ...
def get_user_config(): ...
def send_message(params): ...

class NotificationCenterDelegate(Incomplete):
    def __init__(self) -> None: ...
    def didReceivedNotification(self, id: int, account: int, args: None): ...



import enum
import threading
from _typeshed import Incomplete
from base_plugin import BasePlugin

class DebuggerPlatform(enum.Enum):
    PyCharm = 'pycharm'
    VSCode = 'vscode'

class DebuggerEventListener(BasePlugin):
    id: str
    name: str
    enabled: bool
    initialized: bool
    host: Incomplete
    port: Incomplete
    platform: Incomplete
    def __init__(self, host: str, port: int, platform: DebuggerPlatform) -> None: ...
    def on_app_event(self, event_type: str): ...

class DevServer:
    DEFAULT_HOST: str
    DEFAULT_PORT: int
    SOCKET_TIMEOUT: int
    BUFFER_SIZE: int
    DEBUGGER_DIR: Incomplete
    @classmethod
    def start_server(cls, host: str = None, port: int = None) -> threading.Thread: ...
    @classmethod
    def stop_server(cls) -> bool: ...
    @classmethod
    def setup_remote_debugging(cls, host: str, port: int, platform: DebuggerPlatform) -> bool: ...
    @classmethod
    def stop_remote_debugging(cls, platform: DebuggerPlatform) -> bool: ...



from android.view import View as View
from com.exteragram.messenger.plugins import PluginsConstants
from dataclasses import dataclass, field
from typing import Any, Callable

@dataclass
class Switch:
    key: str
    text: str
    default: bool
    subtext: str = ...
    icon: str = ...
    on_change: Callable[[bool], None] = field(default=None, compare=False, repr=False)
    type: str = field(default=PluginsConstants.Settings.TYPE_SWITCH, init=False)

@dataclass
class Selector:
    key: str
    text: str
    default: int
    items: list[str]
    icon: str = ...
    on_change: Callable[[int], None] = field(default=None, compare=False, repr=False)
    type: str = field(default=PluginsConstants.Settings.TYPE_SELECTOR, init=False)

@dataclass
class Input:
    key: str
    text: str
    default: str = ...
    subtext: str = ...
    icon: str = ...
    on_change: Callable[[str], None] = field(default=None, compare=False, repr=False)
    type: str = field(default=PluginsConstants.Settings.TYPE_INPUT, init=False)

@dataclass
class Text:
    text: str
    icon: str = ...
    accent: bool = ...
    red: bool = ...
    on_click: Callable[[View], None] = field(default=None, compare=False, repr=False)
    create_sub_fragment: Callable[[], list[Any]] = field(default=None, compare=False, repr=False)
    type: str = field(default=PluginsConstants.Settings.TYPE_TEXT, init=False)

@dataclass
class Header:
    text: str
    type: str = field(default=PluginsConstants.Settings.TYPE_HEADER, init=False)

@dataclass
class Divider:
    text: str = ...
    type: str = field(default=PluginsConstants.Settings.TYPE_DIVIDER, init=False)



from _typeshed import Incomplete
from java import jclass as jclass
from typing import Any

class OnClickListener(Incomplete):
    def __init__(self, fn: callable) -> None: ...
    def onClick(self, _view) -> None: ...

class OnLongClickListener(Incomplete):
    def __init__(self, fn: callable) -> None: ...
    def onLongClick(self, _view): ...

def run_on_ui_thread(func: callable, delay: int = 0): ...
def log(data: Any): ...

class R(Incomplete):
    def __init__(self, fn) -> None: ...
    def run(self) -> None: ...



import abc
from abc import ABC, abstractmethod
from dataclasses import dataclass
from de.robv.android.xposed import XC_MethodHook as XC_MethodHook
from enum import Enum
from java.lang.reflect import Member as Member
from typing import Any, Callable, final

@dataclass
class PluginMetadata:
    id: str
    name: str
    description: str
    author: str
    version: str
    icon: str
    min_version: str

class HookStrategy(Enum):
    CANCEL = ...
    MODIFY = ...
    DEFAULT = ...
    MODIFY_FINAL = ...

@dataclass
class HookResult:
    strategy: HookStrategy = ...
    request: Any = ...
    response: Any = ...
    update: Any = ...
    updates: Any = ...
    error: Any = ...
    params: Any = ...

class PluginError(Exception):
    plugin_id: str | None
    def __init__(self, message: str, plugin_id: str | None = None) -> None: ...

class XposedHook(ABC): ...

class MethodReplacement(XposedHook, metaclass=abc.ABCMeta):
    @abstractmethod
    def replace_hooked_method(self, param: XC_MethodHook.MethodHookParam) -> Any: ...

class MethodHook(XposedHook):
    def before_hooked_method(self, param: XC_MethodHook.MethodHookParam): ...
    def after_hooked_method(self, param: XC_MethodHook.MethodHookParam): ...

class AppEvent(Enum):
    START = ...
    STOP = ...
    PAUSE = ...
    RESUME = ...

class MenuItemType(Enum):
    MESSAGE_CONTEXT_MENU = ...
    DRAWER_MENU = ...
    CHAT_ACTION_MENU = ...
    PROFILE_ACTION_MENU = ...

@dataclass
class MenuItemData:
    menu_type: MenuItemType
    text: str
    on_click: Callable[[dict[str, Any]], None]
    item_id: str | None = ...
    icon: str | None = ...
    subtext: str | None = ...
    condition: str | None = ...
    priority: int = ...

class BasePlugin:
    id: str
    name: str
    description: str
    author: str
    min_version: str
    version: str
    icon: str
    error_message: str
    enabled: bool
    initialized: bool
    def __init__(self) -> None: ...
    def on_plugin_load(self) -> None: ...
    def on_plugin_unload(self) -> None: ...
    def create_settings(self) -> list[Any]: ...
    def on_app_event(self, event_type: AppEvent): ...
    def pre_request_hook(self, request_name: str, account: int, request: Any) -> HookResult: ...
    def post_request_hook(self, request_name: str, account: int, response: Any, error: Any) -> HookResult: ...
    def on_update_hook(self, update_name: str, account: int, update: Any) -> HookResult: ...
    def on_updates_hook(self, container_name: str, account: int, updates: Any) -> HookResult: ...
    def on_send_message_hook(self, account: int, params: Any) -> HookResult: ...
    @final
    def add_hook(self, name: str, match_substring: bool = False, priority: int = 0): ...
    @final
    def add_on_send_message_hook(self, priority: int = 0): ...
    @final
    def remove_hook(self, name: str): ...
    @final
    def get_setting(self, key: str, default: Any = None) -> Any: ...
    @final
    def set_setting(self, key: str, value: Any): ...
    @final
    def hook_method(self, method_or_constructor: Member, xposed_hook: Any | None, priority: int | None = None) -> XC_MethodHook.Unhook | None: ...
    @final
    def unhook_method(self, unhook: XC_MethodHook.Unhook): ...
    @final
    def log(self, message: str): ...
    @final
    def add_menu_item(self, menu_item_data: MenuItemData) -> str | None: ...
    @final
    def remove_menu_item(self, item_id: str) -> bool: ...



from dataclasses import dataclass
from enum import Enum

__all__ = ['parse_markdown']

class TLEntityType(Enum):
    CODE = 'code'
    PRE = 'pre'
    STRIKETHROUGH = 'strikethrough'
    TEXT_LINK = 'text_link'
    BOLD = 'bold'
    ITALIC = 'italic'
    UNDERLINE = 'underline'
    SPOILER = 'spoiler'
    CUSTOM_EMOJI = 'custom_emoji'

@dataclass
class RawEntity:
    type: TLEntityType
    offset: int
    length: int
    language: str | None = ...
    url: str | None = ...
    document_id: int | None = ...
    def to_tlrpc_object(self): ...

@dataclass
class ParsedMessage:
    text: str
    entities: tuple[RawEntity, ...]

def parse_markdown(markdown: str) -> ParsedMessage: ...



from _typeshed import Incomplete
from java import jarray as jarray
from typing import Any

JavaClass: Incomplete
JavaObject: Incomplete

def find_class(class_name: str) -> JavaClass | None: ...
def get_private_field(obj: JavaObject, field_name: str) -> Any | None: ...
def set_private_field(obj: JavaObject, field_name: str, new_value: Any) -> bool: ...
def get_static_private_field(clazz: JavaClass, field_name: str) -> Any | None: ...
def set_static_private_field(
    clazz: JavaClass, field_name: str, new_value: Any
) -> bool: ...



from dataclasses import dataclass
from enum import Enum

__all__ = ["parse_markdown"]

class TLEntityType(Enum):
    CODE = "code"
    PRE = "pre"
    STRIKETHROUGH = "strikethrough"
    TEXT_LINK = "text_link"
    BOLD = "bold"
    ITALIC = "italic"
    UNDERLINE = "underline"
    SPOILER = "spoiler"
    CUSTOM_EMOJI = "custom_emoji"

@dataclass
class RawEntity:
    type: TLEntityType
    offset: int
    length: int
    language: str | None = ...
    url: str | None = ...
    document_id: int | None = ...
    def to_tlrpc_object(self): ...
    def __init__(
        self, type, offset, length, language=..., url=..., document_id=...
    ) -> None: ...

@dataclass
class ParsedMessage:
    text: str
    entities: tuple[RawEntity, ...]
    def __init__(self, text, entities) -> None: ...

def parse_markdown(markdown: str) -> ParsedMessage: ...



from _typeshed import Incomplete
from android.content import Context as Context, DialogInterface
from android.graphics.drawable import Drawable as Drawable
from android.view import View as View
from java import Override as Override
from org.telegram.ui.ActionBar import AlertDialog, Theme
from typing import Callable

class _ButtonClickListenerProxy(Incomplete):
    def __init__(self, py_callable: Callable[[AlertDialogBuilder, int], None], builder_instance: AlertDialogBuilder) -> None: ...
    def onClick(self, dialog_java_instance: AlertDialog, which: int): ...

class _ItemsClickListenerProxy(Incomplete):
    def __init__(self, py_callable: Callable[[AlertDialogBuilder, int], None], builder_instance: AlertDialogBuilder) -> None: ...
    def onClick(self, dialog_java_instance: DialogInterface, which: int): ...

class _DismissListenerProxy(Incomplete):
    def __init__(self, py_callable: Callable[[AlertDialogBuilder], None], builder_instance: AlertDialogBuilder) -> None: ...
    def onDismiss(self, dialog_java_instance: DialogInterface): ...

class _CancelListenerProxy(Incomplete):
    def __init__(self, py_callable: Callable[[AlertDialogBuilder], None], builder_instance: AlertDialogBuilder) -> None: ...
    def onCancel(self, dialog_java_instance: DialogInterface): ...

class AlertDialogBuilder:
    ALERT_TYPE_MESSAGE: int
    ALERT_TYPE_LOADING: int
    ALERT_TYPE_SPINNER: int
    BUTTON_POSITIVE: int
    BUTTON_NEGATIVE: int
    BUTTON_NEUTRAL: int
    def __init__(self, context: Context, progress_style: int = ..., resources_provider: Theme.ResourcesProvider | None = None) -> None: ...
    def get_context(self) -> Context: ...
    def set_title(self, title: str) -> AlertDialogBuilder: ...
    def set_message(self, message: str) -> AlertDialogBuilder: ...
    def set_message_text_view_clickable(self, clickable: bool) -> AlertDialogBuilder: ...
    def set_positive_button(self, text: str, listener: Callable[[AlertDialogBuilder, int], None] | None = None) -> AlertDialogBuilder: ...
    def set_negative_button(self, text: str, listener: Callable[[AlertDialogBuilder, int], None] | None = None) -> AlertDialogBuilder: ...
    def set_neutral_button(self, text: str, listener: Callable[[AlertDialogBuilder, int], None] | None = None) -> AlertDialogBuilder: ...
    def make_button_red(self, button_type: int) -> AlertDialogBuilder: ...
    def set_on_back_button_listener(self, listener: Callable[[AlertDialogBuilder, int], None] | None = None) -> AlertDialogBuilder: ...
    def set_view(self, view: View, height: int = -2) -> AlertDialogBuilder: ...
    def set_items(self, items: list[str], listener: Callable[[AlertDialogBuilder, int], None] | None = None, icons: list[int] | None = None) -> AlertDialogBuilder: ...
    def set_on_dismiss_listener(self, listener: Callable[[AlertDialogBuilder], None] | None = None) -> AlertDialogBuilder: ...
    def set_on_cancel_listener(self, listener: Callable[[AlertDialogBuilder], None] | None = None) -> AlertDialogBuilder: ...
    def set_top_image(self, res_id: int, background_color: int) -> AlertDialogBuilder: ...
    def set_top_drawable(self, drawable: Drawable, background_color: int) -> AlertDialogBuilder: ...
    def set_top_animation(self, res_id: int, size: int, auto_repeat: bool, background_color: int, layer_colors: dict[str, int] | None = None) -> AlertDialogBuilder: ...
    def set_top_animation_is_new(self, is_new: bool) -> AlertDialogBuilder: ...
    def set_dim_enabled(self, enabled: bool) -> AlertDialogBuilder: ...
    def set_dialog_button_color_key(self, theme_key: int) -> AlertDialogBuilder: ...
    def set_blurred_background(self, blur: bool, blur_behind_if_possible: bool = True) -> AlertDialogBuilder: ...
    def create(self) -> AlertDialogBuilder: ...
    def show(self) -> AlertDialogBuilder: ...
    def dismiss(self) -> None: ...
    def get_dialog(self) -> AlertDialog | None: ...
    def get_button(self, button_type: int) -> View | None: ...
    def set_progress(self, progress: int): ...
    def set_cancelable(self, cancelable: bool): ...
    def set_canceled_on_touch_outside(self, cancel: bool): ...



from org.telegram.ui.ActionBar import BaseFragment as BaseFragment, Theme as Theme
from typing import Callable

class BulletinHelper:
    DURATION_SHORT: int
    DURATION_LONG: int
    DURATION_PROLONG: int
    @classmethod
    def show_info(cls, message: str, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_error(cls, message: str, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_success(cls, message: str, fragment: BaseFragment | None = None): ...
    @classmethod
    def show_simple(
        cls, text: str, icon_res_id: int, fragment: BaseFragment | None = None
    ): ...
    @classmethod
    def show_two_line(
        cls,
        title: str,
        subtitle: str,
        icon_res_id: int,
        fragment: BaseFragment | None = None,
    ): ...
    @classmethod
    def show_with_button(
        cls,
        text: str,
        icon_res_id: int,
        button_text: str,
        on_click: Callable[[], None] | None,
        fragment: BaseFragment | None = None,
        duration: int = ...,
    ): ...
    @classmethod
    def show_undo(
        cls,
        text: str,
        on_undo: Callable[[], None],
        on_action: Callable[[], None] | None = None,
        subtitle: str | None = None,
        fragment: BaseFragment | None = None,
    ): ...
    @classmethod
    def show_copied_to_clipboard(
        cls, message: str | None = None, fragment: BaseFragment | None = None
    ): ...
    @classmethod
    def show_link_copied(
        cls, is_private_link_info: bool = False, fragment: BaseFragment | None = None
    ): ...
    @classmethod
    def show_file_saved_to_gallery(
        cls,
        is_video: bool = False,
        amount: int = 1,
        fragment: BaseFragment | None = None,
    ): ...
    @classmethod
    def show_file_saved_to_downloads(
        cls,
        file_type_enum_name: str = "UNKNOWN",
        amount: int = 1,
        fragment: BaseFragment | None = None,
    ): ...



from android.view import View as View
from dataclasses import dataclass
from typing import Any, Callable

@dataclass
class Switch:
    key: str
    text: str
    default: bool
    subtext: str = ...
    icon: str = ...
    on_change: Callable[[bool], None] = ...
    type: str = ...
    def __init__(
        self, key, text, default, subtext=..., icon=..., on_change=...
    ) -> None: ...

@dataclass
class Selector:
    key: str
    text: str
    default: int
    items: list[str]
    icon: str = ...
    on_change: Callable[[int], None] = ...
    type: str = ...
    def __init__(self, key, text, default, items, icon=..., on_change=...) -> None: ...

@dataclass
class Input:
    key: str
    text: str
    default: str = ...
    subtext: str = ...
    icon: str = ...
    on_change: Callable[[str], None] = ...
    type: str = ...
    def __init__(
        self, key, text, default=..., subtext=..., icon=..., on_change=...
    ) -> None: ...

@dataclass
class Text:
    text: str
    icon: str = ...
    accent: bool = ...
    red: bool = ...
    on_click: Callable[[View], None] = ...
    create_sub_fragment: Callable[[], list[Any]] = ...
    type: str = ...
    def __init__(
        self, text, icon=..., accent=..., red=..., on_click=..., create_sub_fragment=...
    ) -> None: ...

@dataclass
class Header:
    text: str
    type: str = ...
    def __init__(self, text) -> None: ...

@dataclass
class Divider:
    text: str = ...
    type: str = ...
    def __init__(self, text=...) -> None: ...



import os
import shutil
from pathlib import Path
from setuptools import setup, find_packages
from setuptools.command.install import install
from setuptools.command.develop import develop
from setuptools.command.egg_info import egg_info


def generate_py_from_pyi():
    setup_dir = Path(__file__).parent
    typings_dir = setup_dir / "typings"
    py_stubs_dir = setup_dir / "py_stubs"

    if not typings_dir.exists():
        return

    py_stubs_dir.mkdir(exist_ok=True)

    for pyi_file in typings_dir.rglob("*.pyi"):
        rel_path = pyi_file.relative_to(typings_dir)
        py_path = py_stubs_dir / rel_path.with_suffix(".py")

        py_path.parent.mkdir(parents=True, exist_ok=True)

        shutil.copy2(pyi_file, py_path)


def copy_typing_files(target_dir):
    generate_py_from_pyi()

    setup_dir = Path(__file__).parent
    py_stubs_dir = setup_dir / "py_stubs"

    if not py_stubs_dir.exists():
        print(f"Warning: py_stubs directory not found at {py_stubs_dir}")
        return

    for py_file in py_stubs_dir.rglob("*.py"):
        rel_path = py_file.relative_to(py_stubs_dir)
        target_path = Path(target_dir) / rel_path

        target_path.parent.mkdir(parents=True, exist_ok=True)

        shutil.copy2(py_file, target_path)
        print(f"Copied {rel_path} to {target_path}")


class CustomInstallCommand(install):
    def run(self):
        install.run(self)
        copy_typing_files(self.install_lib)


class CustomDevelopCommand(develop):
    def run(self):
        develop.run(self)
        generate_py_from_pyi()

        setup_dir = Path(__file__).parent
        py_stubs_dir = setup_dir / "py_stubs"
        site_packages = Path(self.install_lib)

        if py_stubs_dir.exists():
            for py_file in py_stubs_dir.rglob("*.py"):
                rel_path = py_file.relative_to(py_stubs_dir)
                target_path = site_packages / rel_path

                target_path.parent.mkdir(parents=True, exist_ok=True)

                if not target_path.exists():
                    target_path.symlink_to(py_file)
                    print(f"Linked {rel_path} to {target_path}")


class CustomEggInfoCommand(egg_info):
    def run(self):
        egg_info.run(self)


setup(
    cmdclass={
        "install": CustomInstallCommand,
        "develop": CustomDevelopCommand,
        "egg_info": CustomEggInfoCommand,
    },
)
